==> atlassian_connect_config-0.3.5.tar.gz: lib/atlassian_connect_config.dart <==
library ac_dart.configuration;

import 'package:atlassian_connect_host/src/preconditions.dart';
import 'package:logging/logging.dart';
import 'package:atlassian_connect_host/host_registration_persistence.dart';
import 'package:atlassian_connect_host/src/host_registration_mongo_persistence.dart';
import 'package:atlassian_connect_host/host_model.dart';
import 'dart:io';

final List<Uri> _autoRegisterHostUrls = [
        Uri.parse("http://localhost:1990/confluence"),
        Uri.parse("http://localhost:2990/jira")
      ];


final Logger _log = new Logger('AcDartConfig');

String _fromEnvironment(String key) {
  final String systemProperty = new String.fromEnvironment(key);
  if (systemProperty != null) {
    return systemProperty;
  }
  
  return Platform.environment[key];
}

int _intFromEnvironment(String key) {
  final String value = _fromEnvironment(key);
  return value == null ? null : int.parse(value);
}

bool _boolFromEnvironment(String key) {
  final String value = _fromEnvironment(key);
  return value == null ? null : value == "true" ? true : false;
}

/// The set of environment properties for AcDart
class AcDartEnvironment {
  static const String AC_ENV = 'AC_ENV';
  static const String AC_ADDON_KEY = 'AC_ADDON_KEY';
  static const String AC_ADDON_SECRET = 'AC_ADDON_SECRET';
  static const String AC_SERVER_PORT = 'PORT';
  static const String AC_BASE_URL = 'AC_BASE_URL';
  static const String AC_MONGO_HOST_URL = 'AC_MONGO_HOST_URL';
  static const String AC_ROOT_LEVEL = 'AC_ROOT_LEVEL';
  static const String AC_LOG_LEVEL = 'AC_LOG_LEVEL';

  // TODO: would be nice to do this for all the env stuff but not sure how to 
  // fit that into the helpers
  // Maybe we could replace the dynamic fromEnvironment(String envKey) with 
  // dynamic fromEnvironment()
  // and implement like () => addonKey etc
  static final String addonKey = _fromEnvironment(AC_ADDON_KEY);
  
  static final String mongoUrlStr = _fromEnvironment(AC_MONGO_HOST_URL);
}

/// The environments that AcDart knows about
class EnvironmentNames {
  static const String development = 'development';
  static const String production = 'production';
}

/**
 * A factory for creating [AcDartConfig] instances. The [standard] constructor
 * supports the model where there are three sets of configuration:
 * 
 * * One common set that acts as default values
 * 
 * * One for development values that override the common values when the 
 * environment is `development`
 * 
 * * One for production values that override the common values when the 
 * environment is `production`
 * 
 * The factory creates an instance of [AcDartConfig] by figuring out which
 * environment is active (based on the AC_ENV environment variable) then merging
 * the config for that environment with the common config
 */
class AcDartConfigFactory {
  final AcDartConfig config;
  final String environmentName;
  bool get isDevelopmentMode => environmentName == EnvironmentNames.development;
  
  AcDartConfigFactory._internal(this.config, this.environmentName);

  factory AcDartConfigFactory.standard(AcDartConfig common, 
      { AcDartConfig development(), AcDartConfig production(), 
        bool defaultEnvToDevelopment: false }) {
    final environmentConfigs = (String environment) {
      switch (environment) {
        case EnvironmentNames.development:
          return development == null ? null : development();
        case EnvironmentNames.production:
          return production == null ? null : production();
        default:
          throw new ArgumentError('unsupported environment value $environment');
      }
    };
    
    return new AcDartConfigFactory.custom(common, environmentConfigs,
        defaultEnvToDevelopment: defaultEnvToDevelopment);
  }
  
  factory AcDartConfigFactory.custom(AcDartConfig common, 
      AcDartConfig environmentConfigs(String environment), 
      { bool defaultEnvToDevelopment: false }) {
    final environment = _stringValue(envKey: AcDartEnvironment.AC_ENV, 
        defaultValue: (defaultEnvToDevelopment ? 
            EnvironmentNames.development : null));
    final environmentConfig = environmentConfigs == null ? 
        null : environmentConfigs(environment);
    final AcDartConfig mergedConfig = environmentConfig == null ? 
        common : common.merge(environmentConfig);
    return new AcDartConfigFactory._internal(mergedConfig, environment);
  }
  
}


/**
 *  A configuration set for an AcDart addon instance.
 *  
 *  The [development] constructor provides defaults suitable for a development
 *  environment. Each of these can be overriden if desired but the aim is to
 *  provide as much defaulting as possible to get started quickly.
 *  
 *  The main constructor is for use for the other configuration sets such as
 *  the common values and production. It provides much less defaulting.
 */
class AcDartConfig {
  final String addonKey;
  final String addonSecret;
  final bool installAddonOnStartup; 
  final int serverPort;
  final String _addonBaseUrlStr;
  Uri get addonBaseUrl => Uri.parse(_addonBaseUrlStr);
  final AcDartHostRepoConfig _repoConfig;
  AcHostRegistrationRepository get acHostRepo => _repoConfig.hostRepo;
  AddonHost get addonHost => new AddonHost(addonKey, addonBaseUrl, addonSecret);

  AcDartConfig(
      { this.addonKey,
        this.serverPort,
        String addonBaseUrlStr,
        this.addonSecret,
        this.installAddonOnStartup: false, // can support passing in hosts to install to
        AcDartHostRepoConfig repoConfig} ) 
        : _addonBaseUrlStr = addonBaseUrlStr,
          _repoConfig = repoConfig;

  factory AcDartConfig.development(
      { String addonKey,
        int serverPort: 9999,
        String addonBaseUrlStr,
        String addonSecret,
        bool installAddonOnStartup: true, 
        AcDartHostRepoConfig repoConfig}) {
    
    final __repoConfig = repoConfig != null ? repoConfig 
        : new AcDartHostRepoConfig.inMemory();
        
    final _addonBaseUrlStr = _baseUrlStr(addonBaseUrlStr, 
        defaultValue: "http://127.0.0.1:$serverPort");
    
    final _addonSecret = addonSecret != null ? addonSecret : 
      'Not very secret ;-). For Development use only!';
    
    return new AcDartConfig(addonKey: addonKey, serverPort: serverPort, 
        addonBaseUrlStr: _addonBaseUrlStr, addonSecret: _addonSecret, 
        installAddonOnStartup: installAddonOnStartup, repoConfig: __repoConfig);
  }
  
  /**
   * Anything in [other] takes precedence. 
   * If no value exists in either then many fields will fallback to an
   * environment variable
   */
  AcDartConfig merge(AcDartConfig other) {
    final _addonKey = _stringValue(baseValue: addonKey, value: other.addonKey, 
        envKey: AcDartEnvironment.AC_ADDON_KEY);     
    final _addonSecret = _stringValue(baseValue: addonSecret, 
        value: other.addonSecret, 
        envKey: AcDartEnvironment.AC_ADDON_SECRET);     
    final _serverPort = _intValue(baseValue: serverPort, value: other.serverPort, 
        envKey: AcDartEnvironment.AC_SERVER_PORT);     
    final __addonBaseUrlStr = _stringValue(baseValue: _addonBaseUrlStr, value: 
      other._addonBaseUrlStr, envKey: AcDartEnvironment.AC_BASE_URL);     
    final _installAddonOnStartup = _boolValue(baseValue: installAddonOnStartup, 
        value: other.installAddonOnStartup, defaultValue: false);
    final _envMongoUrl = _stringValue(envKey: AcDartEnvironment.AC_MONGO_HOST_URL, 
        isOptional: true);
    final __repoConfig = 
        other._repoConfig != null ? other._repoConfig 
          : _repoConfig != null ? _repoConfig 
              : _envMongoUrl != null ? 
                  new AcDartHostRepoConfig.mongo(dbUrl: _envMongoUrl) : null;
    if (__repoConfig == null) {
      throw new ArgumentError("An AcDartHostRepoConfig must be specified in "
          "either the common config or environment config" );
    }
    
    return new AcDartConfig(addonKey: _addonKey, addonSecret: _addonSecret, 
        serverPort: _serverPort, addonBaseUrlStr: __addonBaseUrlStr,
        installAddonOnStartup: _installAddonOnStartup, repoConfig: __repoConfig);
  }
}

/// Configuration for [AcHostRegistrationRepository] instances. Provides
/// convenience constructors for setting up the out of the box implementations
/// (in memory and mongo)
class AcDartHostRepoConfig {
  static final String defaultTestMongoUrlStr = "mongodb://127.0.0.1/acDart";
  final AcHostRegistrationRepositoryFactory _hostRepoFactory;
  AcHostRegistrationRepository get hostRepo => _hostRepoFactory();
  
  AcDartHostRepoConfig(AcHostRegistrationRepositoryFactory hostRepoFactory) 
    : this._hostRepoFactory = checkNotNull(hostRepoFactory, 'hostRepo');
  
  AcDartHostRepoConfig.inMemory() : this(() => 
      new AcHostRegistrationInMemoryTestRepository());
  
  AcDartHostRepoConfig.mongo({String dbUrl, 
        Db db})
    : this(() => new AcHostRegistrationMongoRepository(db != null ? 
        db : new Db(_dbUrl(dbUrl))));
  
  static String _dbUrl(String dbUrl) {
    return dbUrl != null ? dbUrl : AcDartEnvironment.mongoUrlStr;
  }
    
}


dynamic _configValue(dynamic fromEnvironment(String envKey), 
    { dynamic baseValue, dynamic value, String envKey,
      dynamic defaultValue, bool isOptional: false }) {
  if (value != null) {
    return value;
  }
  if (baseValue != null) {
    return baseValue;
  }
  final envValue = envKey == null ? null : fromEnvironment(envKey);
  if (envValue != null) {
    return envValue;
  }
  if (defaultValue != null) {
    return defaultValue;
  }
  if (isOptional) {
    return null;
  }
  throw new ArgumentError("failed to initialse config value. "
      "Environment key: $envKey");
}

String _stringValue({String baseValue, String value, String envKey, 
  String defaultValue, bool isOptional: false }) {
  return _configValue((String envKey) => _fromEnvironment(envKey), 
      baseValue: baseValue, value: value, 
      envKey: envKey, defaultValue: defaultValue, isOptional: isOptional);
}

int _intValue({int baseValue, int value, String envKey, int defaultValue, 
  bool isOptional: false }) {
  return _configValue((String envKey) => _intFromEnvironment(envKey), 
      baseValue: baseValue, value: value, 
      envKey: envKey, defaultValue: defaultValue, isOptional: isOptional);
}

bool _boolValue({bool baseValue, bool value, String envKey, bool defaultValue, 
  bool isOptional: false }) {
  return _configValue((String envKey) => _boolFromEnvironment(envKey), 
      baseValue: baseValue, value: value, 
      envKey: envKey, defaultValue: defaultValue, isOptional: isOptional);
}

// TODO: want to get rid of this but the dev profile is using it for now
String _baseUrlStr(String value, { String defaultValue }) => 
    _stringValue(value: value, envKey: AcDartEnvironment.AC_BASE_URL, 
        defaultValue: defaultValue);

==> atlassian_connect_config-0.4.2.tar.gz: lib/atlassian_connect_config.dart <==
library ac_dart.configuration;

import 'package:atlassian_connect_host/src/preconditions.dart';
import 'package:logging/logging.dart';
import 'package:atlassian_connect_host/host_registration_persistence.dart';
import 'package:atlassian_connect_host/src/host_registration_mongo_persistence.dart';
import 'package:atlassian_connect_host/host_model.dart';
import 'dart:io';
export 'package:atlassian_connect_host/atlassian_connect_host.dart';

final List<Uri> _autoRegisterHostUrls = [
  Uri.parse("http://localhost:1990/confluence"),
  Uri.parse("http://localhost:2990/jira")
];

final Logger _log = new Logger('AcDartConfig');

String _fromEnvironment(String key) {
  final String systemProperty = new String.fromEnvironment(key);
  if (systemProperty != null) {
    return systemProperty;
  }

  return Platform.environment[key];
}

int _intFromEnvironment(String key) {
  final String value = _fromEnvironment(key);
  return value == null ? null : int.parse(value);
}

bool _boolFromEnvironment(String key) {
  final String value = _fromEnvironment(key);
  return value == null ? null : value == "true" ? true : false;
}

/// The set of environment properties for AcDart
class AcDartEnvironment {
  static const String AC_ENV = 'AC_ENV';
  static const String AC_ADDON_KEY = 'AC_ADDON_KEY';
  static const String AC_ADDON_SECRET = 'AC_ADDON_SECRET';
  static const String AC_SERVER_PORT = 'PORT';
  static const String AC_BASE_URL = 'AC_BASE_URL';
  static const String AC_MONGO_HOST_URL = 'AC_MONGO_HOST_URL';
  static const String AC_ROOT_LEVEL = 'AC_ROOT_LEVEL';
  static const String AC_LOG_LEVEL = 'AC_LOG_LEVEL';

  // TODO: would be nice to do this for all the env stuff but not sure how to
  // fit that into the helpers
  // Maybe we could replace the dynamic fromEnvironment(String envKey) with
  // dynamic fromEnvironment()
  // and implement like () => addonKey etc
  static final String addonKey = _fromEnvironment(AC_ADDON_KEY);

  static final String mongoUrlStr = _fromEnvironment(AC_MONGO_HOST_URL);
}

/// The environments that AcDart knows about
class EnvironmentNames {
  static const String development = 'development';
  static const String production = 'production';
}

/**
 * A factory for creating [AcDartConfig] instances. The [standard] constructor
 * supports the model where there are three sets of configuration:
 * 
 * * One common set that acts as default values
 * 
 * * One for development values that override the common values when the 
 * environment is `development`
 * 
 * * One for production values that override the common values when the 
 * environment is `production`
 * 
 * The factory creates an instance of [AcDartConfig] by figuring out which
 * environment is active (based on the AC_ENV environment variable) then merging
 * the config for that environment with the common config
 */
class AcDartConfigFactory {
  final AcDartConfig config;
  final String environmentName;
  bool get isDevelopmentMode => environmentName == EnvironmentNames.development;

  AcDartConfigFactory._internal(this.config, this.environmentName);

  factory AcDartConfigFactory.standard(AcDartConfig common,
      {AcDartConfig development(), AcDartConfig production(),
      bool defaultEnvToDevelopment: false}) {
    final environmentConfigs = (String environment) {
      switch (environment) {
        case EnvironmentNames.development:
          return development == null ? null : development();
        case EnvironmentNames.production:
          return production == null ? null : production();
        default:
          throw new ArgumentError('unsupported environment value $environment');
      }
    };

    return new AcDartConfigFactory.custom(common, environmentConfigs,
        defaultEnvToDevelopment: defaultEnvToDevelopment);
  }

  factory AcDartConfigFactory.custom(
      AcDartConfig common, AcDartConfig environmentConfigs(String environment),
      {bool defaultEnvToDevelopment: false}) {
    final environment = _stringValue(
        envKey: AcDartEnvironment.AC_ENV,
        defaultValue: (defaultEnvToDevelopment
            ? EnvironmentNames.development
            : null));
    final environmentConfig =
        environmentConfigs == null ? null : environmentConfigs(environment);
    final AcDartConfig mergedConfig =
        environmentConfig == null ? common : common.merge(environmentConfig);
    return new AcDartConfigFactory._internal(mergedConfig, environment);
  }
}

/**
 *  A configuration set for an AcDart addon instance.
 *  
 *  The [development] constructor provides defaults suitable for a development
 *  environment. Each of these can be overriden if desired but the aim is to
 *  provide as much defaulting as possible to get started quickly.
 *  
 *  The main constructor is for use for the other configuration sets such as
 *  the common values and production. It provides much less defaulting.
 */
class AcDartConfig {
  final String addonKey;
  final String addonSecret;
  final bool installAddonOnStartup;
  final int serverPort;
  final String _addonBaseUrlStr;
  Uri get addonBaseUrl => Uri.parse(_addonBaseUrlStr);
  final AcDartHostRepoConfig _repoConfig;
  AcHostRegistrationRepository get acHostRepo => _repoConfig.hostRepo;
  AddonHost get addonHost => new AddonHost(addonKey, addonBaseUrl, addonSecret);

  AcDartConfig({this.addonKey, this.serverPort, String addonBaseUrlStr,
      this.addonSecret,
      this.installAddonOnStartup: false, // can support passing in hosts to install to
      AcDartHostRepoConfig repoConfig})
      : _addonBaseUrlStr = addonBaseUrlStr,
        _repoConfig = repoConfig;

  factory AcDartConfig.development({String addonKey, int serverPort: 9999,
      String addonBaseUrlStr, String addonSecret,
      bool installAddonOnStartup: true, AcDartHostRepoConfig repoConfig}) {
    final __repoConfig =
        repoConfig != null ? repoConfig : new AcDartHostRepoConfig.inMemory();

    final _addonBaseUrlStr = _baseUrlStr(addonBaseUrlStr,
        defaultValue: "http://127.0.0.1:$serverPort");

    final _addonSecret = addonSecret != null
        ? addonSecret
        : 'Not very secret ;-). For Development use only!';

    return new AcDartConfig(
        addonKey: addonKey,
        serverPort: serverPort,
        addonBaseUrlStr: _addonBaseUrlStr,
        addonSecret: _addonSecret,
        installAddonOnStartup: installAddonOnStartup,
        repoConfig: __repoConfig);
  }

  /**
   * Anything in [other] takes precedence. 
   * If no value exists in either then many fields will fallback to an
   * environment variable
   */
  AcDartConfig merge(AcDartConfig other) {
    final _addonKey = _stringValue(
        baseValue: addonKey,
        value: other.addonKey,
        envKey: AcDartEnvironment.AC_ADDON_KEY);
    final _addonSecret = _stringValue(
        baseValue: addonSecret,
        value: other.addonSecret,
        envKey: AcDartEnvironment.AC_ADDON_SECRET);
    final _serverPort = _intValue(
        baseValue: serverPort,
        value: other.serverPort,
        envKey: AcDartEnvironment.AC_SERVER_PORT);
    final __addonBaseUrlStr = _stringValue(
        baseValue: _addonBaseUrlStr,
        value: other._addonBaseUrlStr,
        envKey: AcDartEnvironment.AC_BASE_URL);
    final _installAddonOnStartup = _boolValue(
        baseValue: installAddonOnStartup,
        value: other.installAddonOnStartup,
        defaultValue: false);
    final _envMongoUrl = _stringValue(
        envKey: AcDartEnvironment.AC_MONGO_HOST_URL, isOptional: true);
    final __repoConfig = other._repoConfig != null
        ? other._repoConfig
        : _repoConfig != null
            ? _repoConfig
            : _envMongoUrl != null
                ? new AcDartHostRepoConfig.mongo(dbUrl: _envMongoUrl)
                : null;
    if (__repoConfig == null) {
      throw new ArgumentError("An AcDartHostRepoConfig must be specified in "
          "either the common config or environment config");
    }

    return new AcDartConfig(
        addonKey: _addonKey,
        addonSecret: _addonSecret,
        serverPort: _serverPort,
        addonBaseUrlStr: __addonBaseUrlStr,
        installAddonOnStartup: _installAddonOnStartup,
        repoConfig: __repoConfig);
  }
}

/// Configuration for [AcHostRegistrationRepository] instances. Provides
/// convenience constructors for setting up the out of the box implementations
/// (in memory and mongo)
class AcDartHostRepoConfig {
  static final String defaultTestMongoUrlStr = "mongodb://127.0.0.1/acDart";
  final AcHostRegistrationRepositoryFactory _hostRepoFactory;
  AcHostRegistrationRepository get hostRepo => _hostRepoFactory();

  AcDartHostRepoConfig(AcHostRegistrationRepositoryFactory hostRepoFactory)
      : this._hostRepoFactory = checkNotNull(hostRepoFactory, 'hostRepo');

  AcDartHostRepoConfig.inMemory()
      : this(() => new AcHostRegistrationInMemoryTestRepository());

  AcDartHostRepoConfig.mongo({String dbUrl, Db db}) : this(
          () => new AcHostRegistrationMongoRepository(
              db != null ? db : new Db(_dbUrl(dbUrl))));

  static String _dbUrl(String dbUrl) {
    return dbUrl != null ? dbUrl : AcDartEnvironment.mongoUrlStr;
  }
}

dynamic _configValue(dynamic fromEnvironment(String envKey), {dynamic baseValue,
    dynamic value, String envKey, dynamic defaultValue,
    bool isOptional: false}) {
  if (value != null) {
    return value;
  }
  if (baseValue != null) {
    return baseValue;
  }
  final envValue = envKey == null ? null : fromEnvironment(envKey);
  if (envValue != null) {
    return envValue;
  }
  if (defaultValue != null) {
    return defaultValue;
  }
  if (isOptional) {
    return null;
  }
  throw new ArgumentError("failed to initialse config value. "
      "Environment key: $envKey");
}

String _stringValue({String baseValue, String value, String envKey,
    String defaultValue, bool isOptional: false}) {
  return _configValue((String envKey) => _fromEnvironment(envKey),
      baseValue: baseValue,
      value: value,
      envKey: envKey,
      defaultValue: defaultValue,
      isOptional: isOptional);
}

int _intValue({int baseValue, int value, String envKey, int defaultValue,
    bool isOptional: false}) {
  return _configValue((String envKey) => _intFromEnvironment(envKey),
      baseValue: baseValue,
      value: value,
      envKey: envKey,
      defaultValue: defaultValue,
      isOptional: isOptional);
}

bool _boolValue({bool baseValue, bool value, String envKey, bool defaultValue,
    bool isOptional: false}) {
  return _configValue((String envKey) => _boolFromEnvironment(envKey),
      baseValue: baseValue,
      value: value,
      envKey: envKey,
      defaultValue: defaultValue,
      isOptional: isOptional);
}

// TODO: want to get rid of this but the dev profile is using it for now
String _baseUrlStr(String value, {String defaultValue}) => _stringValue(
    value: value,
    envKey: AcDartEnvironment.AC_BASE_URL,
    defaultValue: defaultValue);
